==> warpcore-0.1.4-20150525.121831-2.jar: warpcore/db/core.clj <==
(ns warpcore.db.core
  (:require [warpcore.util.core :as util]
            [datomic.api :as d]
            [taoensso.timbre :as timbre]
            [environ.core :refer [env]]))

(timbre/refer-timbre)

(def ^:dynamic *db-uri* (env :datomic-uri))

(defn conn [] (d/connect *db-uri*))

(defn db [] (d/db (conn)))

;; route helper
(defn wrap-db
  "A Ring middleware that provides a request-consistent database connection and
  value for the life of a request."
  [handler]
  (fn [request]
    (let [conn (conn)]
      (handler (assoc request
                 :conn conn
                 :db (d/db conn))))))

;; creation
(defn load-dtm [conn filename]
  (info "Loading schema file: " filename)
  ;; parse and submit schema transaction
  @(d/transact conn (read-string (slurp (clojure.java.io/resource filename)))))

(defn create! []
  ;; create database
  (info "Creating database" *db-uri*)
  (d/create-database *db-uri*)
  (let [conn (conn)] (load-dtm conn "db/schema.dtm") conn))

;; models

(def meta-props [:uuid :created-at :link-id])

(defn sanitize [props allowed-props & [disallowed-props]]
  (apply dissoc (select-keys props allowed-props)
         (concat meta-props disallowed-props)))

(defn save! [conn entity &
             [{:keys [fill-attrs
                      pull-props
                      blacklist-props
                      whitelist-props
                      temp-id] :as opts}]]
  (let [{:keys [created-at? uuid?]} fill-attrs]
    @(d/transact conn
                 [(merge (if (or whitelist-props blacklist-props)
                           (sanitize entity whitelist-props
                                     (concat meta-props blacklist-props))
                           entity)
                         (util/remove-nil-vals
                          {:db/id (or temp-id (d/tempid :db.part/user))
                           :uuid (if uuid? (d/squuid))
                           :created-at (if created-at? (java.util.Date.))}))])))

(defn save-and-return-full! [conn entity &
                             [{:keys [pull-props] :as opts}]]
  (debug "Creating entity:" entity)
  (let [id (d/tempid :db.part/user)
        {tempids :tempids dba :db-after}
        (save! conn entity (assoc opts :temp-id id))]
    {:db-after dba
     :new-entity
     (d/pull dba (or pull-props "[*]")
             (d/resolve-tempid dba tempids id))}))

(defn save-and-return! [conn entity &
                        [{:keys [pull-props] :as opts}]]
  (:new-entity (save-and-return-full! conn entity opts)))

==> warpcore-0.1.5-20150528.101832-1.jar: warpcore/db/core.clj <==
(ns warpcore.db.core
  (:require [warpcore.util.core :as util]
            [datomic.api :as d]
            [taoensso.timbre :as timbre]
            [environ.core :refer [env]]))

(timbre/refer-timbre)

(def ^:dynamic *db-uri* (env :datomic-uri))

(defn conn [] (d/connect *db-uri*))

(defn db [] (d/db (conn)))

;; route helper
(defn wrap-db
  "A Ring middleware that provides a request-consistent database connection and
  value for the life of a request."
  [handler]
  (fn [request]
    (let [conn (conn)]
      (handler (assoc request
                 :conn conn
                 :db (d/db conn))))))

;; creation
(defn load-dtm [conn filename]
  (info "Loading schema file: " filename)
  ;; parse and submit schema transaction
  @(d/transact conn (read-string (slurp (clojure.java.io/resource filename)))))

(defn create! []
  ;; create database
  (info "Creating database" *db-uri*)
  (d/create-database *db-uri*)
  (let [conn (conn)] (load-dtm conn "db/schema.dtm") conn))

;; models

(def meta-props [:uuid :created-at :link-id])

(defn sanitize [props allowed-props & [disallowed-props]]
  (apply dissoc (select-keys props allowed-props)
         (concat meta-props disallowed-props)))

(defn save! [conn entity &
             [{:keys [fill-attrs
                      pull-props
                      blacklist-props
                      whitelist-props
                      temp-id] :as opts}]]
  (let [{:keys [created-at? uuid?]} fill-attrs]
    @(d/transact conn
                 [(util/remove-nil-vals
                   (merge (if (or whitelist-props blacklist-props)
                            (sanitize entity whitelist-props
                                      (concat meta-props blacklist-props))
                            entity)
                          {:db/id (or temp-id (d/tempid :db.part/user))
                           :uuid (if uuid? (d/squuid))
                           :created-at (if created-at? (java.util.Date.))}))])))

(defn save-and-return-full! [conn entity &
                             [{:keys [pull-props] :as opts}]]
  (debug "Creating entity:" entity)
  (let [id (d/tempid :db.part/user)
        {tempids :tempids dba :db-after}
        (save! conn entity (assoc opts :temp-id id))]
    {:db-after dba
     :new-entity
     (d/pull dba (or pull-props "[*]")
             (d/resolve-tempid dba tempids id))}))

(defn save-and-return! [conn entity &
                        [{:keys [pull-props] :as opts}]]
  (:new-entity (save-and-return-full! conn entity opts)))
